==> streamtuner-0.11.0.tar.gz: streamtuner-0.11.0/src/sgtk/sgtk-dialog.c <==
/*
 * $Id: sgtk-dialog.c,v 1.21 2004/01/05 19:44:09 jylefort Exp $
 *
 * Copyright (c) 2002, 2003 Jean-Yves Lefort
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Jean-Yves Lefort nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <gtk/gtk.h>
#include "sgtk-dialog.h"
#include "sgtk-hig.h"

/*** type definitions ********************************************************/

typedef struct
{
  GtkDialog	*dialog;
  int		response;

  GMutex	*mutex;
  GCond		*cond;
} RunFromThreadInfo;

/*** function declarations ***************************************************/

static void	sgtk_dialog_init	(sGtkDialog		*dialog);
static gboolean	run_from_thread_handler	(RunFromThreadInfo	*info);

/*** implementation **********************************************************/

GType
sgtk_dialog_get_type (void)
{
  static GType dialog_type = 0;
  
  if (! dialog_type)
    {
      static const GTypeInfo dialog_info = {
	sizeof(sGtkDialogClass),
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	sizeof(sGtkDialog),
	0,
	(GInstanceInitFunc) sgtk_dialog_init,
      };
      
      dialog_type = g_type_register_static(GTK_TYPE_DIALOG,
					   "sGtkDialog",
					   &dialog_info,
					   0);
    }

  return dialog_type;
}

static void
sgtk_dialog_init (sGtkDialog *dialog)
{
  int content_area_border;
  int action_area_border;

  /*
   * The kludgy calculations below are the consequence of
   * http://bugzilla.gnome.org/show_bug.cgi?id=98779
   */

  gtk_widget_style_get(GTK_WIDGET(dialog),
		       "content-area-border", &content_area_border,
		       "action-area-border", &action_area_border,
		       NULL);

  gtk_container_set_border_width(GTK_CONTAINER(dialog), SGTK_HIG_DIALOG_BORDER_WIDTH - content_area_border - action_area_border);
  gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog), TRUE);
  gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
  gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), SGTK_HIG_DIALOG_CONTENTS_SPACING - action_area_border - action_area_border);

  dialog->contents = gtk_event_box_new();
  gtk_container_set_border_width(GTK_CONTAINER(dialog->contents), action_area_border);
  gtk_widget_show(dialog->contents);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), dialog->contents, TRUE, TRUE, 0);
}

/*
 * In GTK+ versions prior to 2.2.2 (CVS 20030407), a bug causes
 * gtk_dialog_run() to only work when called from the thread running
 * the GTK+ main loop.
 *
 * To stay compatible with old GTK+ versions, we use this workaround.
 *
 * This function does not work when called from the thread running the
 * GTK+ main loop, and expects to be called while the GDK lock is
 * acquired.
 */
int
sgtk_dialog_run_from_thread (GtkDialog *dialog)
{
  RunFromThreadInfo info;

  g_return_val_if_fail(GTK_IS_DIALOG(dialog), -1);

  info.dialog = dialog;
  info.mutex = g_mutex_new();
  info.cond = g_cond_new();
  info.response = 0;

  /*
   * The GDK lock will be acquired in the main thread by
   * run_from_thread_handler(), so we must release it here and
   * re-acquire it later on.
   */

  GDK_THREADS_LEAVE();

  g_mutex_lock(info.mutex);
  g_idle_add((GSourceFunc) run_from_thread_handler, &info);
  g_cond_wait(info.cond, info.mutex);
  g_mutex_unlock(info.mutex);

  GDK_THREADS_ENTER();

  g_mutex_free(info.mutex);
  g_cond_free(info.cond);
  
  g_return_val_if_fail(info.response != 0, -1);

  return info.response;
}

static gboolean
run_from_thread_handler (RunFromThreadInfo *info)
{
  /*
   * Idle handlers are executed outside the GDK lock, but
   * gtk_dialog_run() expects to be called within the GDK lock, so we
   * must wrap the call.
   */

  GDK_THREADS_ENTER();
  info->response = gtk_dialog_run(info->dialog);
  GDK_THREADS_LEAVE();

  g_cond_signal(info->cond);

  return FALSE;
}

==> streamtuner-0.12.4.tar.gz: streamtuner-0.12.4/src/sgtk/sgtk-dialog.c <==
/*
 * $Id: sgtk-dialog.c,v 1.23 2004/03/08 08:29:56 jylefort Exp $
 *
 * Copyright (c) 2002, 2003 Jean-Yves Lefort
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Jean-Yves Lefort nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <gtk/gtk.h>
#include "sgtk-dialog.h"
#include "sgtk-hig.h"

/*** function declarations ***************************************************/

static void	sgtk_dialog_init	(sGtkDialog		*dialog);

/*** implementation **********************************************************/

GType
sgtk_dialog_get_type (void)
{
  static GType dialog_type = 0;
  
  if (! dialog_type)
    {
      static const GTypeInfo dialog_info = {
	sizeof(sGtkDialogClass),
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	sizeof(sGtkDialog),
	0,
	(GInstanceInitFunc) sgtk_dialog_init,
      };
      
      dialog_type = g_type_register_static(GTK_TYPE_DIALOG,
					   "sGtkDialog",
					   &dialog_info,
					   G_TYPE_FLAG_ABSTRACT);
    }

  return dialog_type;
}

static void
sgtk_dialog_init (sGtkDialog *dialog)
{
  int content_area_border;
  int action_area_border;

  /*
   * The kludgy calculations below are the consequence of
   * http://bugzilla.gnome.org/show_bug.cgi?id=98779
   */

  gtk_widget_style_get(GTK_WIDGET(dialog),
		       "content-area-border", &content_area_border,
		       "action-area-border", &action_area_border,
		       NULL);

  gtk_container_set_border_width(GTK_CONTAINER(dialog), SGTK_HIG_DIALOG_BORDER_WIDTH - content_area_border - action_area_border);
  gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog), TRUE);
  gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
  gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), SGTK_HIG_DIALOG_CONTENTS_SPACING - action_area_border - action_area_border);

  dialog->contents = gtk_event_box_new();
  gtk_container_set_border_width(GTK_CONTAINER(dialog->contents), action_area_border);
  gtk_widget_show(dialog->contents);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), dialog->contents, TRUE, TRUE, 0);
}
